pub struct Node<'a, T> { /* private fields */ }Expand description
A node with an associated weight.
The distribution of points to nodes is proportional to the weights of the nodes. For example, a node with a weight of 3 will receive approximately three times more points than a node with a weight of 1.
Implementations